/*  $Header: /home/cvsroot/dvipdfmx/src/Attic/colorsp.c,v 1.11 2004/08/07 09:52:34 hirata Exp $
    
    This is dvipdfmx, an eXtended version of dvipdfm by Mark A. Wicks.

    Copyright (C) 2002 by Jin-Hwan Cho and Shunsaku Hirata,
    the dvipdfmx project team <dvipdfmx@project.ktug.or.kr>
    
    Copyright (C) 1998, 1999 by Mark A. Wicks <mwicks@kettering.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "system.h"
#include "mem.h"
#include "error.h"

#include "pdfdev.h"
#include "pdfparse.h"
#include "pdfcolor.h"

#include "colorsp.h"

static int
get_values (char **start, char *end, double *values, int count)
{
  char *number;
  int   i;

  for (i = 0; i < count; i++) {
    number = parse_number(start, end);
    if (!number)
      break;
    else {
      values[i] = atof(number);
      RELEASE(number);
    }
  }

  return i;
}

static void
read_colorspec (pdf_color *colorspec, char **start, char *end)
{
  char *token;

  skip_white(start, end);
  token = parse_ident(start, end);
  if (token) {
    if (!strcmp(token, "rgb")) { /* Handle rgb color */
      colorspec->space_id = PDF_COLOR_DEV_RGB;
      get_values(start, end, colorspec->values, 3);
    } else if (!strcmp(token, "cmyk")) { /* Handle cmyk color */
      colorspec->space_id = PDF_COLOR_DEV_CMYK;
      get_values(start, end, colorspec->values, 4);
    } else if (!strcmp(token, "gray")) { /* Handle gray */
      colorspec->space_id = PDF_COLOR_DEV_GRAY;
      get_values(start, end, colorspec->values, 1);
    } else if (!strcmp(token, "hsb")) {
      WARN("color special: hsb not implemented.");
    } else { /* Must be a "named" color */
      pdf_color_by_name(colorspec, token);
    }
    RELEASE(token);
  }
}

static void
do_color_special (char **start, char *end)
{
  pdf_color colorspec;

  skip_white(start, end);
  if (*start < end) {
    if (*start + 4 <= end &&
	!strncmp(*start, "push", 4)) { /* Handle a push */ 
      *start += 4;
      read_colorspec(&colorspec, start, end);
      pdf_color_stack_push(&colorspec);
    } else if (*start + 3 <= end &&
	       !strncmp(*start, "pop", 3)) { /* Handle a pop */
      *start += 3;
      pdf_color_stack_pop();
    } else { /* Assume this is a default color change */
      read_colorspec(&colorspec, start, end);
      pdf_color_set_default(&colorspec);
      dev_do_color(); /* .... */
    }
  }
}

static void
do_background_special (char **start, char *end)
{
  pdf_color colorspec;

  read_colorspec(&colorspec, start, end);
  pdf_color_set_bgcolor(&colorspec);
}

/*
 * The two specials papersize and landscape are applied only if
 * they were in the first page of the DVI file. Otherwise ignored.
 * See dvi_paper_size() in dvi.c for more details. */

/* Color_special() handles color specials and also
   handles papersize and landscape specials since
   there's no good place to put them  */

int
color_special (char *buffer, UNSIGNED_QUAD size)
{
  char *start = buffer, *end;
  int result = 0;

  end = buffer + size;
  skip_white(&start, end);
  if (!strncmp(start, "color", strlen("color"))) {
    start += strlen("color");
    result = 1; /* This is a color special */
    do_color_special(&start, end);
  } else if (!strncmp(start, "background", strlen("background"))) {
    start += strlen("background");
    result = 1; /* This is a color special */
    do_background_special(&start, end);
  } else if (!strncmp(start, "landscape", strlen("landscape"))) {
    result = 1;
  } else if (!strncmp(start, "papersize", strlen("papersize"))) {
    result = 1; 
  }

  return result;
}
